import { prefix } from "./utils";
import { Direction } from "./direction";

export class ExnihiloBlock {
    readonly Namespace: string;
    private create: Function;

    constructor(namespace: string) {
        this.Namespace = prefix(namespace);
    }

    protected setCreate(create: Function) {
        this.create = create;
    }

    protected addComponent(name: string, value: any) {
        this.create(value, `minecraft:block/components/${name}`)
    }

    protected addBlockTag(namespace: string) {
        this.addComponent(`tag:${namespace}`, {});
    }

    protected setGeometry(name: string) {
        this.addComponent(`minecraft:geometry`, `geometry.${name}`)
    }

    protected setDestroyTime(time: number) {
        this.addComponent(`minecraft:destructible_by_mining`, { seconds_to_destroy: time });
    }

    protected setExplosionResistance(hardness: number) {
        this.addComponent(`minecraft:destructible_by_explosion`, { explosion_resistance: hardness })
    }

    protected setFlammable(catch_chance: number, destroy_chance: number) {
        this.addComponent(`minecraft:flammable`, {
            catch_chance_modifier: catch_chance,
            destroy_chance_modifier: destroy_chance
        })
    }

    protected addBlockProperty(name: string, values: string[] | number[] | boolean[]) {
        this.create(values, `minecraft:block/description/properties/${prefix(name)}`);
    }

    protected addPartVisibility(bone_name: string[], condition: string[]) {
        bone_name.forEach((v, i) => {
            this.addComponent(`minecraft:part_visibility/conditions`, { [v]: condition[i] })
        })
    }

    protected addMaterialInstances(direction: Direction[], material: { [key: string]: string | boolean }[]) {
        this.create(this.getMaterialInstances(direction, material), `minecraft:block/components`);
    }

    protected getMaterialInstances(direction: Direction[], material: { [key: string]: string | boolean }[]) {
        let o = { "minecraft:material_instances": {} };
        direction.forEach((v, i) => {
            o["minecraft:material_instances"][v] = material[i];
        });
        return o;
    }

    protected addEvent(name: string, value: {}) {
        this.create(value, `minecraft:block/events/${name}`);
        return { [name]: value };
    }

    protected addPermutation(name: string, value: {}) {
        this.create(value, `minecraft:block/permutations/${name}`);
        return { [name]: value };
    }
}