import { Block, BlockLocation, BlockPermutation, Dimension, world } from "@minecraft/server";
import { logerr, prefix } from "../utils";

export class BlockBase {
    public block: Block;
    public location: BlockLocation;
    public dimension: Dimension;
    public permutation: BlockPermutation;

    constructor(block: Block) {
        this.block = block;
        this.location = this.block.location;
        this.dimension = this.block.dimension;
        this.permutation = this.block.permutation;
    }

    static setNonPushable(identifier: string) {
        return world.events.beforePistonActivate.subscribe(arg => {
            for (let location of arg.piston.attachedBlocks) {
                let block = arg.block.dimension.getBlock(location).typeId;
                if (block == identifier || block.includes(identifier)) {
                    arg.cancel = true;
                    return;
                }
            }
        })
    }

    public getBlockProperty(name: string): string | number | boolean {
        if (!name.includes(":")) name = prefix(name);
        let permutation = this.permutation.getProperty(name);
        // @ts-ignored
        if (permutation) return permutation.value;
    }

    public setBlockProperty(name: string, value: string | number | boolean) {
        if (!name.includes(":")) name = prefix(name);
        if (this.permutation.getProperty(name) != null) {
            // @ts-ignored
            this.permutation.getProperty(name).value = value;
        }
    }

    public commitSetPermutation() {
        try {
            this.block.setPermutation(this.permutation);
        } catch (e) {
            logerr(e)
        }
    }

    public setBlockProperties(properties: { [name: string]: string | number | boolean }) {
        for (let name in properties) {
            this.setBlockProperty(name, properties[name]);
        }

        this.commitSetPermutation();
    }

    public hasBlockProperty(properties: { [name: string]: string | number | boolean }): boolean {
        for (let name in properties) {
            let matchValue = properties[name];
            if (!name.includes(":")) name = prefix(name);
            let value = this.getBlockProperty(name);
            // @ts-ignored
            if (value == null || value != matchValue) return false;
        }
        return true;
    }
}