import {
    Location,
    BlockLocation,
    Dimension,
    CommandResult,
    world,
    InventoryComponentContainer,
    Player,
    IEntityComponent,
    EntityInventoryComponent
} from '@minecraft/server';

const NAMESPACE = "exnihilo";

function prefix(s: string): string {
    return `${NAMESPACE}:${s}`;
}

function locationToString(location: Location | BlockLocation | { x: number, y: number, z: number }): string {
    return `${location.x} ${location.y} ${location.z}`;
}

function playSoundAt(
    soundId: string,
    location: Location | BlockLocation | { x: number, y: number, z: number },
    dimension: Dimension,
    options: { volume: number, pitch: number } = { volume: null, pitch: null }): Promise<CommandResult> {
    return dimension.runCommandAsync(`playsound ${soundId} @a ${locationToString(location)} ${notNull(options.volume)} ${notNull(options.pitch)}`);
}

/**
 * Return value if it is not null else return defaultValue or empty string
 */
function notNull(value: any, defaultValue: any = ""): any {
    return value !== null ? value : defaultValue;
}

function splitNamespace(s: string): { "isVanilla": boolean, "first": string, "last": string } {
    let l = s.split(":")
    return {
        "isVanilla": l[0] == "minecraft",
        "first": l[0],
        "last": l[1]
    };
}

function getPlayerInventory({ entity }): InventoryComponentContainer {
    if (!(entity instanceof Player)) throw new Error("Entity isn't a player")
    let playerInventory: IEntityComponent = entity.getComponent("inventory")
    if (!(playerInventory instanceof EntityInventoryComponent)) throw new Error("Player doesn't have inventory")
    return playerInventory.container
}

function logobj(o: Object) {
    Object.getOwnPropertyNames(Object.getPrototypeOf(o)).forEach(v => {
        try {
            world.say(`${v}: ${o[v]}`)
        } catch (e) {
            world.say(`${v}: ${e}`)
        }
    })
}

function logerr(e: Error) {
    world.say(`${e}\n${e.stack}`)
}

export {
    locationToString,
    playSoundAt,
    notNull,
    prefix,
    splitNamespace,
    logobj,
    logerr,
    getPlayerInventory
};